// Channel A: 1 is blue, 2 is green      Channel B: 4 is black, 3 is red
#define IRpin_PIN      PIND
#define IRpin          2

#include "pitches.h"

// notes in the melody:
int melody[] = {
  NOTE_C4, NOTE_G3,NOTE_G3, NOTE_A3, NOTE_G3,0, NOTE_B3, NOTE_C4};

// note durations: 4 = quarter note, 8 = eighth note, etc.:
int noteDurations[] = {
  4, 8, 8, 4,4,4,4,4 };

int speedTurn = 3;
int speedDrive = 11;
int dirTurn = 12;
int dirDrive = 13;

int speedState = 0;

// the maximum pulse we'll listen for - 65 milliseconds is a long time
#define MAXPULSE 6500
#define NUMPULSES 50

// what our timing resolution should be, larger is better
// as its more 'precise' - but too large and you wont get
// accurate timing
#define RESOLUTION 20 

// What percent we will allow in variation to match the same code
#define FUZZINESS 50

// we will store up to 100 pulse pairs (this is -a lot-)
uint16_t pulses[NUMPULSES][2];  // pair is high and low pulse 
uint8_t currentpulse = 0; // index for pulses we're storing

#include "ircodes1.h"

void setup(void) {
pinMode(speedTurn, OUTPUT);
pinMode(speedDrive, OUTPUT);
pinMode(dirTurn, OUTPUT);
pinMode(dirDrive, OUTPUT);
}

void loop(void) {
  int numberpulses;
  
  numberpulses = listenForIR();
  
  if (IRcompare(numberpulses, OnOff,sizeof(OnOff)/4)) {
    Serial.println("On/Off");
  }
    if (IRcompare(numberpulses, Right,sizeof(Right)/4)) {
    turnRight();
  }
    if (IRcompare(numberpulses, Left,sizeof(Left)/4)) {
    turnLeft();
  }
    if (IRcompare(numberpulses, Forward,sizeof(Forward)/4)) {
    goForward();
  }
    if (IRcompare(numberpulses, Reverse,sizeof(Reverse)/4)) {
    goReverse();
  }
    if (IRcompare(numberpulses, Break,sizeof(Break)/4)) {
    breakSpeed(); 
  }
    if (IRcompare(numberpulses, leftArrow,sizeof(leftArrow)/4)) {
    special1(); 
  }
    
  delay(100);
}

//KGO: added size of compare sample. Only compare the minimum of the two
boolean IRcompare(int numpulses, int Signal[], int refsize) {
  int count = min(numpulses,refsize);
  for (int i=0; i< count-1; i++) {
    int oncode = pulses[i][1] * RESOLUTION / 10;
    int offcode = pulses[i+1][0] * RESOLUTION / 10;
    
#ifdef DEBUG    
    Serial.print(oncode); // the ON signal we heard
    Serial.print(" - ");
    Serial.print(Signal[i*2 + 0]); // the ON signal we want 
#endif   
    
    // check to make sure the error is less than FUZZINESS percent
    if ( abs(oncode - Signal[i*2 + 0]) <= (Signal[i*2 + 0] * FUZZINESS / 100)) {
#ifdef DEBUG
      Serial.print(" (ok)");
#endif
    } else {
#ifdef DEBUG
      Serial.print(" (x)");
#endif
      // we didn't match perfectly, return a false match
      return false;
    }
    
    
#ifdef DEBUG
    Serial.print("  \t"); // tab
    Serial.print(offcode); // the OFF signal we heard
    Serial.print(" - ");
    Serial.print(Signal[i*2 + 1]); // the OFF signal we want 
#endif    
    
    if ( abs(offcode - Signal[i*2 + 1]) <= (Signal[i*2 + 1] * FUZZINESS / 100)) {
#ifdef DEBUG
      Serial.print(" (ok)");
#endif
    } else {
#ifdef DEBUG
      Serial.print(" (x)");
#endif
      // we didn't match perfectly, return a false match
      return false;
    }
    
#ifdef DEBUG
    Serial.println();
#endif
  }
  // Everything matched!
  return true;
}

int listenForIR(void) {
  currentpulse = 0;
  
  while (1) {
    uint16_t highpulse, lowpulse;  // temporary storage timing
    highpulse = lowpulse = 0; // start out with no pulse length
  
//  while (digitalRead(IRpin)) { // this is too slow!
    while (IRpin_PIN & (1 << IRpin)) {
       // pin is still HIGH

       // count off another few microseconds
       highpulse++;
       delayMicroseconds(RESOLUTION);

       // If the pulse is too long, we 'timed out' - either nothing
       // was received or the code is finished, so print what
       // we've grabbed so far, and then reset
       
       // KGO: Added check for end of receive buffer
       if (((highpulse >= MAXPULSE) && (currentpulse != 0))|| currentpulse == NUMPULSES) {
         return currentpulse;
       }
    }
    // we didn't time out so lets stash the reading
    pulses[currentpulse][0] = highpulse;
  
    // same as above
    while (! (IRpin_PIN & _BV(IRpin))) {
       // pin is still LOW
       lowpulse++;
       delayMicroseconds(RESOLUTION);
        // KGO: Added check for end of receive buffer
        if (((lowpulse >= MAXPULSE)  && (currentpulse != 0))|| currentpulse == NUMPULSES) {
         return currentpulse;
       }
    }
    pulses[currentpulse][1] = lowpulse;

    // we read one high-low pulse successfully, continue!
    currentpulse++;
  }
}

void turnLeft() {
  digitalWrite(dirTurn, LOW);
  analogWrite(speedTurn, 255);
  delay(30);
  analogWrite(speedTurn, 0);
  
}

void turnRight() {
  digitalWrite(dirTurn, HIGH);
  analogWrite(speedTurn, 255);
  delay(30);
  analogWrite(speedTurn, 0);  
}

void goForward() {
  digitalWrite(dirDrive, LOW);
  analogWrite(speedDrive, 150);
}

void goReverse() {
  digitalWrite(dirDrive, HIGH);
  analogWrite(speedDrive, 150);
}

void breakSpeed() {
  analogWrite(speedDrive, 0);
}

void special1() {
   // iterate over the notes of the melody:
  for (int thisNote = 0; thisNote < 8; thisNote++) {

    // to calculate the note duration, take one second 
    // divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000/noteDurations[thisNote];
    tone(8, melody[thisNote],noteDuration);

    // to distinguish the notes, set a minimum time between them.
    // the note's duration + 30% seems to work well:
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    // stop the tone playing:
    noTone(8);
 }
}
  
